SystemOrganization addCategory: #TiledMorph!
SystemOrganization addCategory: #'TiledMorph-Tests'!

Object subclass: #ImageSlicer
	instanceVariableNames: 'image rows columns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph'!

!ImageSlicer class methodsFor: 'helpers' stamp: 'SebastianNozzi 10/7/2012 11:20'!
crop: form forTilingIn: columns by: rows
	| widthToReduce heightToReduce reductionLeft reductionRight reductionTop reductionBottom croppingBounds |
	widthToReduce := form width \\ columns.
	heightToReduce := form height \\ columns.
	reductionLeft := (widthToReduce / 2) truncated * (-1).
	reductionRight := (widthToReduce / 2) rounded * (-1).
	reductionTop := (heightToReduce / 2) truncated * (-1).
	reductionBottom := (heightToReduce / 2) rounded * (-1).
	croppingBounds := form boundingBox expandBy: ((reductionLeft@reductionTop) corner: (reductionRight@reductionBottom)).
	^ form copy: croppingBounds.! !

!ImageSlicer class methodsFor: 'instance creation' stamp: 'SebastianNozzi 10/7/2012 11:37'!
new: columns by: rows forFileNamed: fileName
	| form |
	form := self crop: (Form fromFileNamed: fileName) forTilingIn: columns by: rows.
	^ self new
		image: form;
		columns: columns;
		rows: rows;
		yourself! !

!ImageSlicer methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/5/2012 17:59'!
boundsOfTileAt: tilePosition
	| tileExtent |
	tileExtent := self tileExtent.
	^ (tileExtent * (tilePosition - (1 @ 1))) corner: (tileExtent * tilePosition)! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:49'!
columns
	^ columns! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:49'!
columns: amountOfColumns
	columns := amountOfColumns! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:43'!
image
	^ image! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:43'!
image: aForm
	image := aForm
! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:46'!
imageExtent
	^ image extent! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:45'!
imageHeight
	^ self imageExtent y
! !

!ImageSlicer methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/5/2012 19:41'!
imageSliceAt: tilePosition
	| bounds | 
	bounds := self boundsOfTileAt: tilePosition.
	(bounds height isInteger and: [bounds width isInteger]) ifFalse: [self error: 'Image can not be split'].
	^ image copy: bounds.! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:45'!
imageWidth
	^ self imageExtent x
! !

!ImageSlicer methodsFor: 'initialize-release' stamp: 'SebastianNozzi 10/5/2012 17:44'!
initialize 
	rows := 3.
	columns := 3.! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:49'!
rows
	^ rows! !

!ImageSlicer methodsFor: 'accessing' stamp: 'SebastianNozzi 10/5/2012 17:49'!
rows: amountOfRows
	rows := amountOfRows
! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:48'!
tileExtent
	^ self imageExtent / (columns@rows)
! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:48'!
tileHeight
	^ self tileExtent y
! !

!ImageSlicer methodsFor: 'querying' stamp: 'SebastianNozzi 10/5/2012 17:48'!
tileWidth
	^ self tileExtent x
! !

Object subclass: #Tile
	instanceVariableNames: 'parent bgColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph'!

!Tile commentStamp: 'SebastianNozzi 9/29/2012 21:03' prior: 0!
I am the superclass for all kinds of Tiles. I am added at a certain position to a TiledMorph.

Attachment
Position
Event-Handling
!

Tile subclass: #ImageTile
	instanceVariableNames: 'imgForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph'!

!ImageTile commentStamp: 'SebastianNozzi 9/29/2012 20:59' prior: 0!
I draw an image (Form) in my tile area. I do not stretch the image, so be sure it is of the right dimension.

If you want to save memory, you can share the same Form between many instances of me.!

!ImageTile class methodsFor: 'instance creation' stamp: 'SebastianNozzi 9/27/2012 17:06'!
newFromFileNamed: fileName
	^ self newWithForm: (Form fromFileNamed: fileName)! !

!ImageTile class methodsFor: 'instance creation' stamp: 'SebastianNozzi 9/27/2012 16:58'!
newWithForm: aForm
	^ self new
		image: aForm;
		yourself! !

!ImageTile methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:17'!
drawOn: aCanvas
	super drawOn: aCanvas.
	self image
		ifNotNil: [ 
			aCanvas translucentImage:  self image at: self topLeftCoordinates ].! !

!ImageTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/25/2012 17:02'!
image
	^ imgForm! !

!ImageTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:12'!
image: aForm
	imgForm := aForm.
	self changed.! !

Tile subclass: #LetterTile
	instanceVariableNames: 'letter font color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph'!

!LetterTile commentStamp: 'SebastianNozzi 9/29/2012 20:59' prior: 0!
I draw a single letter in the middle of the tile area.
You can change my font, color and background color.

Be sure not to set a string, but a SINGLE CHARACTER.!

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:07'!
color
	^ color.! !

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:12'!
color: aColor
	color := aColor.
	self changed.
! !

!LetterTile methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:02'!
drawLetterOn: aCanvas
	aCanvas
		drawString: self letter asString
		at: self letterCoordinates
		font: self font
		color: self color! !

!LetterTile methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:15'!
drawOn: aCanvas
	super drawOn: aCanvas.
	self drawLetterOn: aCanvas.! !

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:07'!
font
	^ font.! !

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:12'!
font: aFont
	font := aFont.
	self changed.
! !

!LetterTile methodsFor: 'initialize' stamp: 'SebastianNozzi 9/29/2012 12:16'!
initialize 
	super initialize.
	font := StrikeFont familyName: 'Arial' size: 72.
	color := Color black.
	letter := $A.! !

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 01:08'!
letter
	^ letter.! !

!LetterTile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 20:58'!
letter: aCharacter
	self assert: (aCharacter isKindOf: Character) description: 'Must be Character'.
	letter := aCharacter.
	self changed.
! !

!LetterTile methodsFor: 'private-coordinates' stamp: 'SebastianNozzi 9/29/2012 12:03'!
letterCoordinates
	|  centerOfTile  |
	centerOfTile := self centerOfTile.
	^ ((centerOfTile x - ((font widthOf: (self letter)) / 2)) @ (centerOfTile y - (font height / 2))) asIntegerPoint
! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:29'!
adjacentPositions
	^ {(self leftPosition).
	(self rightPosition).
	(self upPosition).
	(self downPosition)} select: [ :position | parent hasTilePosition: position ]! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:30'!
adjacentTiles
	^ self adjacentPositions collect: [:position | parent tileAt: position].! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 18:54'!
attachTo: aTiledMorph
	parent := aTiledMorph.! !

!Tile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 12:15'!
bgColor
	^ bgColor ! !

!Tile methodsFor: 'accessing' stamp: 'SebastianNozzi 9/29/2012 12:15'!
bgColor: aColor
	bgColor := aColor.
	self changed.! !

!Tile methodsFor: 'private-coordinates' stamp: 'SebastianNozzi 9/29/2012 12:05'!
centerOfTile
	^ self tileBounds center.! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/29/2012 01:12'!
changed
	self ifAttached: [parent tileChanged: self].! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 18:56'!
detach.
	parent := nil.! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:29'!
downPosition 
	^ self positionPlusOffset: 0@1.! !

!Tile methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:16'!
drawBackgroundOn: aCanvas
	bgColor ifNotNil: [aCanvas fillRectangle: self tileBounds color: bgColor]! !

!Tile methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:17'!
drawOn: aCanvas
	self drawBackgroundOn: aCanvas.! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 19:22'!
ensureAttached
	self isAttached ifFalse: [self error: 'Tile is not attached'].! !

!Tile methodsFor: 'event-handling' stamp: 'SebastianNozzi 9/29/2012 20:52'!
handleKeystroke: anEvent! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 19:23'!
ifAttached: block
	^ self isAttached ifTrue:[block value].! !

!Tile methodsFor: 'initialize' stamp: 'SebastianNozzi 9/29/2012 12:16'!
initialize
	parent := nil.
	bgColor := nil.! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:31'!
isAdjacentTo: anotherTile
	^ self adjacentTiles includes: anotherTile ! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 19:22'!
isAttached
	^ parent isNil not! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/29/2012 16:03'!
isNotAttached
	^ parent isNil ! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:29'!
leftPosition 
	^ self positionPlusOffset: (-1)@0.! !

!Tile methodsFor: 'event-handling' stamp: 'SebastianNozzi 9/29/2012 20:42'!
mouseDown: anEvent 
! !

!Tile methodsFor: 'position' stamp: 'SebastianNozzi 10/4/2012 17:43'!
moveTo: newPosition
	self ensureAttached.
	parent at: newPosition putTile: self! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 17:25'!
parent
	^ parent! !

!Tile methodsFor: 'private-parent' stamp: 'SebastianNozzi 9/27/2012 17:25'!
parent: aTiledMorph
	parent := aTiledMorph
! !

!Tile methodsFor: 'position' stamp: 'SebastianNozzi 9/27/2012 19:17'!
position
	self ensureAttached.
	^ parent positionOfTile: self.
! !

!Tile methodsFor: 'private-querying' stamp: 'SebastianNozzi 10/4/2012 17:29'!
positionPlusOffset: anOffset
	^ self position + anOffset ! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:29'!
rightPosition 
	^ self positionPlusOffset: 1@0.! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 19:31'!
tileAbove
	^ self parent tileAt: self upPosition.  ! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 19:31'!
tileBelow
	^ self parent tileAt: self downPosition.  ! !

!Tile methodsFor: 'querying' stamp: 'SebastianNozzi 9/29/2012 11:46'!
tileBounds
	self ensureAttached.
	^ parent boundsOfTile: self.! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 19:31'!
tileToTheLeft
	^ self parent tileAt: self leftPosition.  ! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 19:31'!
tileToTheRight 
	^ self parent tileAt: self rightPosition.  ! !

!Tile methodsFor: 'querying' stamp: 'SebastianNozzi 9/29/2012 12:00'!
topLeftCoordinates
	^ self tileBounds topLeft! !

!Tile methodsFor: 'querying-positions' stamp: 'SebastianNozzi 10/4/2012 17:29'!
upPosition 
	^ self positionPlusOffset: 0@(-1).! !

Morph subclass: #TiledMorph
	instanceVariableNames: 'borderColor borderWidth rowCount columnCount rowHeight columnWidth tiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph'!

!TiledMorph commentStamp: 'SebastianNozzi 9/29/2012 21:03' prior: 0!
I am a Morph that show tiles. You can put instances of class Tile in my positions.

Dimensions, event-handling, etc.

| t |
t := TiledMorph new: 3 by: 3.
t openInWorld.
t at: 1@1 putTile: Tile new.!

!TiledMorph class methodsFor: 'instance-creation' stamp: 'SebastianNozzi 9/2/2012 14:49'!
new: amountOfColumns by: amountOfRows
	^ (self new) 
		initializeWithColumns: amountOfColumns  andRows: amountOfRows;
		yourself.! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/29/2012 22:25'!
allTilePositions
	| allPositions |
	allPositions := OrderedCollection new.
	(1 to: (self rowCount)) do: [:row |
		(1 to: (self columnCount)) do: [:col | 
			allPositions add: (col@row).
			]].
	^ allPositions.
! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/16/2012 12:16'!
allTilePositionsDo: blockAcceptingPosition
	self tilePositions: (self allTilePositions) do: blockAcceptingPosition.! !

!TiledMorph methodsFor: 'accessing-tiles' stamp: 'SebastianNozzi 9/27/2012 18:54'!
at: position putTile: tile
	self removeTileAt: position.
	self removeTile: tile.
	tiles at: position put: tile.
	tile attachTo: self.
	self changed! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 20:00'!
backgroundColor 
	^ self color! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 20:00'!
backgroundColor: newColor
	self color: newColor.! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 16:13'!
borderColor
	^ borderColor! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 19:54'!
borderColor: newColor
	borderColor := newColor.
	self changed.! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/29/2012 11:45'!
boundsOfTile: tile
	^ self rectangleForTileAtPosition: (self positionOfTile: tile).
! !

!TiledMorph methodsFor: 'accessing' stamp: 'SebastianNozzi 8/30/2012 16:05'!
columnCount
	^ columnCount! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:06'!
columnWidth
	^ columnWidth ! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:15'!
columnWidth: width
	columnWidth := width.
	self updateDimensions.! !

!TiledMorph methodsFor: 'private-layout' stamp: 'SebastianNozzi 8/30/2012 22:08'!
desiredHeight
	^ ((self rowHeight+borderWidth) * self rowCount + borderWidth)! !

!TiledMorph methodsFor: 'private-layout' stamp: 'SebastianNozzi 8/30/2012 22:08'!
desiredWidth
	^ ((self columnWidth + borderWidth) * self columnCount + borderWidth)! !

!TiledMorph methodsFor: 'drawing' stamp: 'SebastianNozzi 8/30/2012 16:09'!
drawBackgroundOn: aCanvas
	aCanvas fillRectangle: (self bounds) color: (self backgroundColor).
! !

!TiledMorph methodsFor: 'drawing' stamp: 'SebastianNozzi 9/3/2012 18:23'!
drawColumnLinesOn: aCanvas
	(0 to: self columnCount) do: [:i || xCoord  |
		xCoord := (self left + (i*(self columnWidth + (borderWidth/1)))).
		aCanvas line: (xCoord @ (self top)) to: (xCoord @ (self bottom)) width: borderWidth color: borderColor.
		].! !

!TiledMorph methodsFor: 'drawing' stamp: 'SebastianNozzi 9/27/2012 20:04'!
drawOn: aCanvas
	self drawBackgroundOn: aCanvas.
	
	self showingBorders ifTrue: [
		self drawRowLinesOn: aCanvas.
		self drawColumnLinesOn: aCanvas].

	self drawTilesOn: aCanvas.
		! !

!TiledMorph methodsFor: 'drawing' stamp: 'SebastianNozzi 9/16/2012 13:07'!
drawRowLinesOn: aCanvas
	(0 to: self rowCount) do: [:i || yCoord  |
		yCoord := (self top + (i*(self rowHeight + borderWidth))).
		aCanvas line: ((self left) @yCoord) to: ((self right) @ yCoord) width: borderWidth color: borderColor.
		]! !

!TiledMorph methodsFor: 'drawing' stamp: 'SebastianNozzi 9/29/2012 12:04'!
drawTilesOn: aCanvas
	self allTilePositionsDo: [:tilePosition |
		self tileAt: tilePosition ifNotNilDo: [:tile |  
			tile drawOn: aCanvas.
		]].! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/16/2012 12:20'!
fillWithTilesOfClass: tileClass
	self allTilePositionsDo: [:pos | self at: pos putTile: (tileClass new)].
	self changed.! !

!TiledMorph methodsFor: 'event handling' stamp: 'SebastianNozzi 9/29/2012 20:52'!
handleKeystroke: anEvent
	| eventPosition | 
	super handleKeystroke: anEvent.
	eventPosition := World activeHand position .
	self tileAtScreenCoordinates: eventPosition ifNotNilDo: [ :tileWhenKeyPressed | self handleKeystroke:  anEvent onTile: tileWhenKeyPressed ].
! !

!TiledMorph methodsFor: 'event handling' stamp: 'SebastianNozzi 9/29/2012 20:52'!
handleKeystroke:  anEvent onTile: tileWhenKeyPressed
	tileWhenKeyPressed handleKeystroke: anEvent.
! !

!TiledMorph methodsFor: 'private-events' stamp: 'SebastianNozzi 8/30/2012 19:34'!
handlesMouseDown: anEvent
	^ true! !

!TiledMorph methodsFor: 'private-events' stamp: 'SebastianNozzi 8/30/2012 19:27'!
handlesMouseOver: anEvent
	^ true! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/29/2012 20:36'!
hasTilePosition: tilePosition
	 ^ (tilePosition >= (1@1)) and: [tilePosition <= (columnCount @ rowCount )]! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 9/3/2012 18:32'!
hideBorders
	self showBorders: false.! !

!TiledMorph methodsFor: 'initialize' stamp: 'SebastianNozzi 9/3/2012 18:30'!
initialize
	self initializeWithColumns: 3 andRows: 3.! !

!TiledMorph methodsFor: 'initialize' stamp: 'SebastianNozzi 8/30/2012 19:59'!
initializeColors
	self color: Color lightGray.
	borderColor := Color gray.
! !

!TiledMorph methodsFor: 'initialize' stamp: 'SebastianNozzi 9/27/2012 09:54'!
initializeTiles
	tiles := Dictionary new.! !

!TiledMorph methodsFor: 'initialize' stamp: 'SebastianNozzi 9/29/2012 20:03'!
initializeWithColumns: amountOfColumns andRows: amountOfRows
	super initialize.
	rowCount := amountOfRows.
	columnCount := amountOfColumns.
	rowHeight := 20.
	columnWidth := 20.
	self initializeTiles.
	self showBorders: true.
	self initializeColors.
	self updateDimensions.! !

!TiledMorph methodsFor: 'event handling' stamp: 'SebastianNozzi 9/29/2012 20:48'!
mouseDown: anEvent
	super mouseDown: anEvent.
	self tileAtScreenCoordinates: anEvent position ifNotNilDo: [ :clickedTile | self mouseDown: anEvent onTile: clickedTile ]! !

!TiledMorph methodsFor: 'event handling' stamp: 'SebastianNozzi 9/29/2012 20:48'!
mouseDown: anEvent onTile: clickedTile
	clickedTile mouseDown: anEvent.! !

!TiledMorph methodsFor: 'private-events' stamp: 'SebastianNozzi 8/30/2012 19:27'!
mouseEnter: anEvent
	anEvent hand newKeyboardFocus: self! !

!TiledMorph methodsFor: 'private-events' stamp: 'SebastianNozzi 9/2/2012 20:46'!
mouseLeave: anEvent
	"anEvent hand newKeyboardFocus: nil"! !

!TiledMorph methodsFor: 'accessing-tiles' stamp: 'SebastianNozzi 9/27/2012 19:18'!
positionOfTile: tile
	^ tiles keyAtValue: tile ifAbsent: [self error: 'Tile not found'].! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/25/2012 18:43'!
realCoordinatesForPosition: tilePosition
	^ (self left + (((tilePosition x) - 1) * (self columnWidth + borderWidth) + borderWidth)) 
	@ (self top + (((tilePosition y) - 1) * ( self rowHeight + borderWidth) + borderWidth))! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/25/2012 18:49'!
rectangleForTileAtPosition: tilePosition
	^ (self realCoordinatesForPosition: tilePosition) extent: (self tileExtent).! !

!TiledMorph methodsFor: 'accessing-tiles' stamp: 'SebastianNozzi 9/27/2012 18:52'!
removeTile: tile
	(tiles keyAtValue: tile ifAbsent: [ nil ]) ifNotNilDo: [ :oldPosition | self removeTileAt: oldPosition ].
! !

!TiledMorph methodsFor: 'accessing-tiles' stamp: 'SebastianNozzi 9/27/2012 18:52'!
removeTileAt: position
	self tileAt: position ifNotNilDo: [:tile | tile detach. tiles removeKey: position. self changed].! !

!TiledMorph methodsFor: 'accessing' stamp: 'SebastianNozzi 8/30/2012 16:05'!
rowCount
	^ rowCount! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:06'!
rowHeight
	^ rowHeight! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:12'!
rowHeight: height
	rowHeight := height.
	self updateDimensions.! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 9/3/2012 18:32'!
showBorders
	self showBorders: true.! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 22:06'!
showBorders: trueOrFalse
	borderWidth := (trueOrFalse ifTrue: [ 1] ifFalse: [0]).
	self updateDimensions.
	self changed.! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 8/30/2012 22:12'!
showingBorders
	^ borderWidth > 0! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 21:57'!
squareTileSize: size
	self rowHeight: size.
	self columnWidth: size.! !

!TiledMorph methodsFor: 'actions' stamp: 'SebastianNozzi 10/4/2012 17:43'!
swapTile: firstTile with: secondTile
	| tmpPosition |
	tmpPosition := firstTile position.
	self at: (secondTile position) putTile: firstTile.
	self at: tmpPosition putTile: secondTile.! !

!TiledMorph methodsFor: 'accessing-tiles' stamp: 'SebastianNozzi 9/16/2012 11:59'!
tileAt: position
	^ tiles at: position ifAbsent: [nil]! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/16/2012 13:09'!
tileAt: tilePosition ifNotNilDo: blockAcceptingTile
	self tileAt: tilePosition ifNotNilDo: blockAcceptingTile ifNil: [].! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/16/2012 13:00'!
tileAt: tilePosition ifNotNilDo: blockWithTile ifNil: block
	| tile |
	tile := self tileAt: tilePosition.
	tile ifNil: block ifNotNil: [ blockWithTile value: tile ].! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/29/2012 20:55'!
tileAtScreenCoordinates: absoluteCoordinates ifNotNilDo: blockAcceptingTile
	| tilePosition |
	tilePosition := self tilePositionAtScreenCoordinates: absoluteCoordinates.
	(self hasTilePosition: tilePosition)
		ifTrue: [ 
			self tileAt: tilePosition ifNotNilDo: blockAcceptingTile ]! !

!TiledMorph methodsFor: 'private-layout' stamp: 'SebastianNozzi 9/29/2012 01:12'!
tileChanged: aTile
	self changed.! !

!TiledMorph methodsFor: 'visual-properties' stamp: 'SebastianNozzi 9/25/2012 18:27'!
tileExtent
	^ self columnWidth @ self rowHeight! !

!TiledMorph methodsFor: 'accessing-dimensions' stamp: 'SebastianNozzi 9/29/2012 21:57'!
tileExtent: extent
	self rowHeight: extent y.
	self columnWidth: extent x.! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/12/2012 09:00'!
tilePositionAtMorphCoordinates: coordinatesWithinBounds
	| x y |
	x := ((coordinatesWithinBounds x) / (self columnWidth + borderWidth)) truncated.	
	y := ((coordinatesWithinBounds y) / (self rowHeight + borderWidth)) truncated.
	^ (x+1)@(y+1)! !

!TiledMorph methodsFor: 'querying-coordinates' stamp: 'SebastianNozzi 9/12/2012 09:14'!
tilePositionAtScreenCoordinates: screenCoordinates
	^ self tilePositionAtMorphCoordinates: (screenCoordinates - (self topLeft)).! !

!TiledMorph methodsFor: 'enumerating-tiles' stamp: 'SebastianNozzi 9/16/2012 12:17'!
tilePositions: positions do: blockAcceptingPosition
	positions do: [:tilePosition | blockAcceptingPosition value: tilePosition].! !

!TiledMorph methodsFor: 'private-layout' stamp: 'SebastianNozzi 8/30/2012 15:53'!
updateDimensions
	self width: (self desiredWidth).
	self height: (self desiredHeight).! !

TestCase subclass: #TiledMorphTest
	instanceVariableNames: 'tile tiledMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph-Tests'!

!TiledMorphTest methodsFor: 'running' stamp: 'SebastianNozzi 9/29/2012 16:01'!
setUp 
	tile := Tile new.
	tiledMorph := TiledMorph new: 3 by: 3.! !

!TiledMorphTest methodsFor: 'tests-attachment' stamp: 'SebastianNozzi 9/29/2012 16:01'!
testAttachment
	tiledMorph at: 1@1 putTile: tile.
	self assert: tile isAttached.! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:11'!
testDesiredHeight
	tiledMorph rowHeight: 30.
	self assert: tiledMorph desiredHeight equals: 94.! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:13'!
testDesiredWidth
	tiledMorph columnWidth: 40.
	self assert: tiledMorph desiredWidth equals: 124.! !

!TiledMorphTest methodsFor: 'tests-attachment' stamp: 'SebastianNozzi 9/29/2012 16:02'!
testDetachedAfterRemove
	tiledMorph at: 1@1 putTile: tile.
	self assert: tile isAttached.
	tiledMorph removeTile: tile.
	self assert: tile isNotAttached.
	! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:34'!
testHasTilePosition
	self assert: (tiledMorph hasTilePosition: -12@47) not.
	self assert: (tiledMorph hasTilePosition: 0@0) not.
	self assert: (tiledMorph hasTilePosition: 1@1).
	self assert: (tiledMorph hasTilePosition: 2@2).
	self assert: (tiledMorph hasTilePosition: 3@3).
	self assert: (tiledMorph hasTilePosition: 4@5) not.
! !

!TiledMorphTest methodsFor: 'tests-attachment' stamp: 'SebastianNozzi 9/29/2012 16:03'!
testIsNotAttached
	self assert: tile isNotAttached.! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:17'!
testMorphExtent
	tiledMorph rowHeight: 30.
	tiledMorph columnWidth: 40.
	self assert: tiledMorph extent equals: 124@94.! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:02'!
testSettingDimensions
	tiledMorph rowHeight: 30.
	tiledMorph columnWidth: 40.
	self assert: 30 equals: tiledMorph rowHeight.
	self assert: 40 equals: tiledMorph columnWidth.
	! !

!TiledMorphTest methodsFor: 'tests-dimensions' stamp: 'SebastianNozzi 9/29/2012 20:17'!
testTileExtent
	tiledMorph rowHeight: 30.
	tiledMorph columnWidth: 40.
	self assert: tiledMorph tileExtent equals: 40@30.! !
